PRO restele_batch,day,type

; RESTORATION of images (2D arrays) for
; the 2-D theoretical MTF of a diffraction-limited telescope.
; This version is used to perform massive restoration of
; many frames, integer, normalized to 10000.
; (The restored arrays must have xdim, ydim = even number)
; Used to restore the SST 2003 data.
; 
; INPUTS: day   = date (16,19,20)
;	  type  = channel ('gc', 'c6')
; 	Pre-processed images (*pp*) in sav2 format
;	Logfiles
; OUTPUT: Integer frames (*rr*) in sav2 format,
;	  numbered according to logfile
;
; Note: program asks for Wiener filter parameters.
; In case of constant S/N, C=15-30 seems to be adequate.
; In case of variable S/N the approximation is:
; 	1/mtf(f)   for f le fo (in arcsec^-1)
;	mtf(f)/(mtf(f)^2+K*(f-fo))   for f gt fo
; i.e., increasing K we do more filtering.
;
; Routines called: NINT, MEAN, RESTORE2, SPC, FFT(IDL), SWAP_ENDIAN(IDL)
;
; Version of RESTELE.PRO, 4 March 2004, Michal


on_error,1

; PARAMETERS OF RESTORATION ---------------------------------------------

	stp=0.041         ; SCALE (PIXEL SIZE) OF THE IMAGE  (arcsec/pix)
	  if type eq 'gc' then  $
	lambda=4.507d-5   else  $ 
	lambda=6.020d-5	  ; WORKING WAVELENGTH (cm)
	d=97.		  ; DIAMETER OF THE TELESCOPE (cm)

; IMAGE SIZE (to be given properly to create the correct filter)

	xdim=1024
	ydim=1024

; PATHS AND FILENAMES

	sday=strtrim(fix(day),2)
	dsday=sday
	if day eq 19 then begin
	  print,'Process series (s) or single frames (other keys) ?'
	  if get_kbrd(1) eq 's' then dsday='19ser'
	endif

	pa1='/home/msobotka/data/sst03/'
	pa2=type+'_'+dsday+'/'		;paths
	ifil=type+'_'+sday+'pp.'	;input name
	ofil=type+'_'+sday+'rr.'	;output name

; read logfile
	logname=findfile(pa1+pa2+'*lg*.sav2')
	restore2,logname,log,/swap
	nfr=(size(log))(2)
	print,'Number of frames:',nfr


; END OF EDITABLE PARAMETERS --------------------------------------------

n=xdim>ydim	; taking larger of the dimensions, so that
		; the basic working frame size will be n,n

a=findgen(n/2+1)/(n*stp)                   ; scale array for plotting

; 1-D calculation of MTF ------------------------------------------------

mtf=dblarr(n)
cc=lambda*2.062648d5/(d*n*stp)   ; 206264.8 arcsec in radian
ss=0

   for i=1,n do begin
	      
      omega=cc*(i-1)
      if omega lt 1.d0 then begin
        tt=acos(omega)-omega*sqrt(1.-omega*omega)
      endif else begin
        ss=ss+1
	if ss eq 1 then print,'MTF cutoff: ',i-1
	tt=0.d0
      endelse
      mtf(i-1)=2.*tt/3.141592654d0

   endfor

window,2,ys=640
plot,a,mtf,linestyle=5,xra=[-0.2,12],yra=[-0.2,4],/xst,/yst
print,''

; creating the 1-D Wiener filter ----------------------------------------

newc:
print,'Wiener filter with constant S/N (=0) or variable (=1) ?'
if get_kbrd(1) eq '0' then begin
	read,'Filter parameter C = ',c
	t1=(c+1)*mtf/(c*mtf*mtf+1)
endif else begin
	read,'Filter parameters K, fo : ',k,fo
	fp=nint(fo*n*stp)	; conversion to pixels
	t1=dblarr(n)		; filter
	uu=findgen(n-fp-1)+1.	; linearly increasing N/S
	uu=uu/n/stp
	t1(0:fp)=1./mtf(0:fp)
	t1(fp+1:n-1)=mtf(fp+1:n-1)/(mtf(fp+1:n-1)^2+k*uu)
endelse

wset,2
plot,a,t1,xra=[-0.2,12],yra=[-0.2,4],/xst,/yst,/noerase
print,'Change filter? (y/n)'
if get_kbrd(1) eq 'y' then goto,newc

print,'Do you want to stop here? (y/n)'
if get_kbrd(1) eq 'y' then begin
	out=0
	goto,fin
endif
wdelete,2

read,'Width of apodized edges in % of image size: ',perc

t1=float(t1)

; transition to 2-D Wiener filter t -------------------------------------

t=fltarr(n,n)
f1=fltarr(n/2+1,n/2+1)

cros0=min(where(t1 le 0))		; zero crossing point of t1,
					; i.e. we can compute only
for j=0,cros0 do begin			; the non-zero part. The rest
	for i=0,cros0 do begin		; are zeros by definition.
		ii=float(i)
		jj=float(j)
		r=sqrt(ii*ii+jj*jj)
		r1=fix(r)
		if r1 ge cros0 then begin
		  f1(i,j)=0.
		endif else begin
		  f1(i,j)=t1(r1)+(t1(r1+1)-t1(r1))*(r-r1)
		endelse
	endfor
endfor

f2=reverse(f1(1:n/2-1,*),1)

t(0:n/2,0:n/2)=f1			; t - 2-D Wiener filter
t(n/2+1:n-1,0:n/2)=f2
t(n/2+1:n-1,n/2+1:n-1)=reverse(f2(*,1:n/2-1),2)
t(0:n/2,n/2+1:n-1)=reverse(f1(*,1:n/2-1),2)
f1=0
f2=0
print,'   2-D Wiener filter created'

; creating apodization window apw ----------------------------------------

   npx=nint(xdim*perc/100.)
   npy=nint(ydim*perc/100.)

   ux=fltarr(xdim)+1.		; x-bell
   ux(0)=0.
   for i=1,npx do ux(i)=(1.-cos(!pi*i/npx))/2.
   ux(xdim-npx:xdim-1)=reverse(ux(1:npx))

   uy=fltarr(ydim)+1.		; y-bell
   uy(0)=0.
   for i=1,npy do uy(i)=(1.-cos(!pi*i/npy))/2.
   uy(ydim-npy:ydim-1)=reverse(uy(1:npy))

   apw=ux#uy		; apodization window (xdim,ydim)
   print,'   Apodization window created'


; RESTORATION ------------------------------------------------------------

for i=0,nfr-1 do begin	; Loop over frames

; Reading input frame

  num=string(fix(log(0,i)),format='(I4.4)')	;frame number
  name=pa1+pa2+ifil+num
  restore2,name,ima
  si=size(ima)
  if (si(1) ne xdim) or (si(2) ne ydim) then  $
	message,'!!! wrong image size !!!'

; apodization

   mea=mean(ima)
   ima=ima-mea
   ima=ima*apw
   ima=ima+mea	 	; apodized frame
  
; creating working array (n,n)

   im=fltarr(n,n)+mea	; working array with mean(ima)
   im(0,0)=ima		; insert image

; 2-D Fourier filtering
   
   f1=fft(im,-1)
   out=float(fft(f1*t,1))   ; restored frame (real part)
   out=nint(out(0:xdim-1,0:ydim-1)) ; back to original size and type

; Storing restored frame

  print,'Storing: ',pa2+ofil+num,i+1
  save2,pa1+pa2+ofil+num,out

endfor	; End loop over frames

fin:
print,'End of program.'

END

